/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008 <talsi.elderme@gmail.com>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

const float pi = 3.14159265;

varying vec2 direction;

uniform vec3 sunDirection;
uniform sampler2D skyMap;
uniform sampler2D skyNightMap;


void main (void) {

	vec2 texCoord;
	vec2 texCoordNight = gl_TexCoord[0].st;

	vec3 dir;

	dir.x = cos(direction.y)*sin(direction.x);
	dir.y = sin(direction.y);
	dir.z = cos(direction.y)*cos(direction.x);

	float cosAngle = dot(sunDirection,dir);

	texCoord.s = acos(cosAngle)/pi;
	texCoord.t = direction.y *2./pi;

	vec3 skyColor = texture2D(skyMap, texCoord).xyz;
	vec3 nightColor = texture2D(skyNightMap, texCoordNight).xyz;

	float alpha = sunDirection.y + (sunDirection.y+0.1)*smoothstep(0.,1.,(1.+cosAngle)/2.);

	alpha = clamp(alpha,0.,1.);
	vec3 color = alpha * skyColor + (1.-alpha) * nightColor;


	//fog
	float fog = smoothstep(0.,pi/10., direction.y);
	fog = clamp(fog, 0., 1.);
	color = mix(vec3(gl_Fog.color), color, fog);


	gl_FragColor = vec4(color, 1.);
}


