/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gameChooserKeyboardOptions.h"


GameChooserKeyboardOptions::GameChooserKeyboardOptions(std::string fileName) :
	KeyboardOptions (fileName,
					 (KeyMap[]) {{SDLK_UP,"forward_key"},{SDLK_DOWN,"back_key"},{SDLK_LEFT,"left_key"},{SDLK_RIGHT,"right_key"},{SDLK_SPACE,"launch_key"}},5) {
}
	
GameChooserKeyboardOptions::~GameChooserKeyboardOptions() {	
}


void GameChooserKeyboardOptions::writeConfigurationFile() {
	std::ofstream file(mFileName.c_str(), std::ios::trunc);
	
	if ( !file ){
		std::cerr << "Cannot create Game Chooser keyboard configuration file." << std::endl;
		exit(1);
	}
	
	std::vector<std::string> comments;
	
	file << "## Newt-Puzzle Game Chooser keyboard options file. \n \n";
	
	comments.push_back("Key binding for moving forward.");
	file << writeParam("forward_key", comments);
	comments.clear();
	
	comments.push_back("Key binding for moving back.");
	file << writeParam("back_key", comments);
	comments.clear();	
	
	comments.push_back("Key binding for moving to the left.");
	file << writeParam("left_key", comments);
	comments.clear();
	
	comments.push_back("Key binding for moving to the right.");
	file << writeParam("right_key", comments);
	comments.clear();

	comments.push_back("Key binding for launching a game.");
	file << writeParam("launch_key", comments);
	comments.clear();	
	
	file.close();		
}




