/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 

#include "window.h"
 
static SDL_Surface *screen = NULL;

static bool quit = false; 
static bool gamePaused = false;
 
static float elapsedTime = 0.;

static bool keyRepeat = false;

static int xMouse;
static int yMouse;
static bool mouseLeft = false;
static bool mouseMiddle = false;
static bool mouseRight = false;
static int mouseWheel = 0;
static bool cursorMove = false;
static bool cursorMoveBackup = false;

 
void window_printSDLGLinfos() {
	int red,green,blue,alpha,info;
    SDL_GL_GetAttribute(SDL_GL_DOUBLEBUFFER, &info);
	cout << "Double buffer " << info << "\n";
	
	SDL_GL_GetAttribute(SDL_GL_BUFFER_SIZE,        &info);
    SDL_GL_GetAttribute(SDL_GL_RED_SIZE,           &red);
    SDL_GL_GetAttribute(SDL_GL_GREEN_SIZE,         &green);
    SDL_GL_GetAttribute(SDL_GL_BLUE_SIZE,          &blue);
    SDL_GL_GetAttribute(SDL_GL_ALPHA_SIZE,         &alpha);
	cout << "Frame buffer "<< info << ", red=" << red  << ",green=" << green << ", blue=" << blue<< ", alpha=" << alpha << "\n";
	
    SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE, &info);
	cout << "Depth buffer " << info << "\n";
	
    SDL_GL_GetAttribute(SDL_GL_MULTISAMPLEBUFFERS, &info);
	cout << "Multisample " << info << "\n";
    SDL_GL_GetAttribute(SDL_GL_MULTISAMPLESAMPLES, &info);
	cout << "Number of samples " << info << "\n";
	
    SDL_GL_GetAttribute(SDL_GL_ACCUM_RED_SIZE, &red);
    SDL_GL_GetAttribute(SDL_GL_ACCUM_GREEN_SIZE, &green);
    SDL_GL_GetAttribute(SDL_GL_ACCUM_BLUE_SIZE, &blue);
    SDL_GL_GetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, &alpha);
	cout << "Accum buffer : red=" << red << ", green=" << green << ", blue=" << blue<< ", alpha=" << alpha << "\n";	
}
 
void* window_loadGLextension(const char *fname) {
	return SDL_GL_GetProcAddress(fname);
}
 

int getTime() {
	return SDL_GetTicks();
}

 
bool window_init( int *argc, char **argv, char *window_title, char *icon_title) {
	
    if (SDL_Init(SDL_INIT_VIDEO) != 0) {
		cout << "Cannot initialize SDL \n" << SDL_GetError() <<"\n" ;
		return false;
	}
		
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, generalGraphics->isDoubleBuffer());
	
	if(!generalGraphics->isSystemAntiAliasingParamEnable()) {
		window_setAntiAliasing(generalGraphics->getNbAntiAliasing());
	}
	
	if(generalGraphics->isFullscreen()) {
		screen = SDL_SetVideoMode(generalGraphics->getScreenWidth(),
								  generalGraphics->getScreenHeight(),
								  generalGraphics->getBppMode(),
								  SDL_OPENGL | SDL_FULLSCREEN);		
	} else {
		screen = SDL_SetVideoMode(generalGraphics->getScreenWidth(),
								  generalGraphics->getScreenHeight(),
								  generalGraphics->getBppMode(),
								  SDL_OPENGL);
	}
	
	if (screen == NULL) {
		cout << "Cannot set SDL video mode \n" << SDL_GetError() <<"\n" ;
		return false;		
	}
	
	SDL_WM_SetCaption( window_title, icon_title );

	SDL_EnableUNICODE(1);

	window_warpPointer(generalGraphics->getScreenWidth()/2,generalGraphics->getScreenHeight()/2);

	if(!generalGraphics->isSystemAnisotropyParamEnable()) {
		setAnisotropicFiltering(generalGraphics->getNbAnisotropy());
	}
	
	glHint (GL_GENERATE_MIPMAP_HINT, GL_NICEST);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	
	
	//window_printSDLGLinfos();
	//printGLInfos();

	return true;
}

void window_quit() {
	quit = true;
}


void window_exit() {

	generalGraphics->writeConfigurationFile();
	generalKeyboardOptions->writeConfigurationFile();
	newt->getNewtCurrentState()->getKeyboardOptions()->writeConfigurationFile();
	
	int seconds, minuts, hours;
	seconds = (int) elapsedTime;
	hours = seconds / 3600;
	seconds -= hours * 3600;
	minuts = seconds / 60;
	seconds -= minuts * 60;
	
	ostringstream oss;
	oss << "You have wasted ";
	
	if(hours) {
		oss << hours << "h " << minuts << "m " << seconds << "s.";
	} else {
		if(minuts) {
			oss << minuts << "m " << seconds << "s.";
		} else {
			oss << seconds << "s. ;)";
		}
	}
	
	std::cout << oss.str() << std::endl;
	
	SDL_Quit();
}


void window_setAntiAliasing(int nbSample) {
	if (SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1) == -1) {
		cout << "Cannot activate anti-aliasing \n";
	} else {
		if (SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, nbSample) == -1) {
			cout << "Cannot initialize SDL_GL_MULTISAMPLESAMPLES \n";
		}
	}
}


void window_enable_key_repeat(bool enabled) {
    if ( enabled ) {
		SDL_EnableKeyRepeat( SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL );
    } else {
		SDL_EnableKeyRepeat( 0, 0 );
    }
	keyRepeat = enabled;
}

bool keyboardEvent(SDLKey key, double timeInterval) {	
	
	if(key == generalKeyboardOptions->getKey("pause_key")) {
		/*gamePaused = !gamePaused;
		if(gamePaused) {
			cursorMoveBackup = cursorMove;			
			cursorMove = true;
		} else {
			cursorMove = cursorMoveBackup;
		}*/
		return false;
	}
	
	if(key == generalKeyboardOptions->getKey("screenshot_key")) {
		screenshot(newt->getNewtCurrentState()->getName());
		return false;
	}	
	
	return newt->keyboardEvent(key, timeInterval);
}


 
void window_warpPointer(int x, int y){
    SDL_WarpMouse( x, y );
}

void window_showCursor(bool show) {
    SDL_ShowCursor(show);
}

void window_captureCursor(bool captureCursor) {
	cursorMove = !captureCursor;
}


void window_event(bool isActive) {

	SDL_Event event; 
	std::set<SDLKey> keyboardInput;
	
	
	/* for mouse */
	int xOld = generalGraphics->getScreenWidth()/2;
	int yOld = generalGraphics->getScreenHeight()/2;
	xMouse = xOld;
	yMouse = yOld;
	
	double currentTime;
	double lastTime;
	double deltaTime;
	
	int n = 0;
	double intervalTime = 0.;

	bool keyPressed = false;
		
	while( quit == false ) {
				
		while( SDL_PollEvent( &event )) {			
			switch (event.type) {
				case SDL_QUIT:
            		quit = true;
					break;

	    		case SDL_ACTIVEEVENT:
					/* Don't draw scene if the window has been minimized */
					if ((event.active.gain == 0)&&(event.active.state & SDL_APPACTIVE)) {
		  				isActive = false;
					} else {
		  				isActive = true;
					}
					break;				

	    		case SDL_KEYDOWN:
					keyboardInput.insert(event.key.keysym.sym);
					keyPressed = true;
					break;	
				
        		case SDL_KEYUP:	
					keyPressed = false;
					break;
						
				case SDL_MOUSEMOTION:
					xOld = xMouse;
					yOld = yMouse;
					SDL_GetMouseState(&xMouse, &yMouse);
					newt->getNewtCurrentState()->mouseMotion(xOld, yOld, xMouse, yMouse, mouseLeft, mouseMiddle, mouseRight);

					break;
						
	    		case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == SDL_BUTTON_LEFT)
						mouseLeft = true;
					if (event.button.button == SDL_BUTTON_MIDDLE)
						mouseMiddle = true;
					if (event.button.button == SDL_BUTTON_RIGHT)
						mouseRight = true;
					if (event.button.button == SDL_BUTTON_WHEELDOWN)
						mouseWheel = -1;
					if (event.button.button == SDL_BUTTON_WHEELUP)
						mouseWheel = 1;					
					break;
					
	    		case SDL_MOUSEBUTTONUP:
					if (event.button.button == SDL_BUTTON_LEFT) {
						newt->getNewtCurrentState()->leftClick(xMouse,yMouse);	
						mouseLeft = false;
					}
					if (event.button.button == SDL_BUTTON_MIDDLE) {
						newt->getNewtCurrentState()->middleClick(xMouse,yMouse);	
						mouseMiddle = false;
					}
					if (event.button.button == SDL_BUTTON_RIGHT) {
						newt->getNewtCurrentState()->rightClick(xMouse,yMouse);	
						mouseRight = false;
					}
					if (event.button.button == SDL_BUTTON_WHEELDOWN) {
						mouseWheel = 0;
						newt->getNewtCurrentState()->mouseWheel(-1);	
					}
					if (event.button.button == SDL_BUTTON_WHEELUP) {
						mouseWheel = 0;	
						newt->getNewtCurrentState()->mouseWheel(1);							
					}
						
					break;
						
	
					
				default:
					break;
            }
		}
		
		if (isActive) {
			
			if((!cursorMove)&&(!gamePaused)) {
				window_warpPointer(generalGraphics->getScreenWidth()/2,generalGraphics->getScreenHeight()/2);
				xOld = generalGraphics->getScreenWidth()/2;
				yOld = generalGraphics->getScreenHeight()/2;
				xMouse = xOld;
				yMouse = yOld;
			}
			
			lastTime = currentTime;
			currentTime = (double)SDL_GetTicks () * 0.001f;
			deltaTime = currentTime - lastTime;
			elapsedTime += deltaTime;
			
			for(std::set<SDLKey>::iterator it = keyboardInput.begin();it!=keyboardInput.end();it++) {
				bool repeatable = keyboardEvent(*it, deltaTime);
				if(!repeatable) {
					keyboardInput.erase(*it);
				}
			}
			if(!keyPressed) {
				keyboardInput.clear();
			} 
			
			if(!gamePaused) {
				
				NewtState *newtState = newt->getNewtCurrentState();
				
				if(newtState->haveToStopWastingTime()) {
					newt->quitCurrentNewtState();
				} else {
					newtState-> updateAnimations(currentTime, intervalTime);	
					scene->render();
				}

				n++;
				intervalTime += (currentTime - lastTime);
				if ((intervalTime) >= 3.0) {
					n /=3;
					std::cout << "fps: " << n << std::endl;
					n = 0;
					intervalTime = 0.;
				}
				
				glFlush();
				printGLError();
				SDL_GL_SwapBuffers ();
			}
		} 
		

		//SDL_Delay(1);
		
	
	}
	
	window_exit();

}
